from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.user_skill_entry import UserSkillEntry


T = TypeVar("T", bound="SkillListResponse")


@_attrs_define
class SkillListResponse:
    """Response for listing skills.

    Attributes:
        skills (list[UserSkillEntry]):
        total (int):
    """

    skills: list[UserSkillEntry]
    total: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        skills = []
        for skills_item_data in self.skills:
            skills_item = skills_item_data.to_dict()
            skills.append(skills_item)

        total = self.total

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "skills": skills,
                "total": total,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_skill_entry import UserSkillEntry

        d = dict(src_dict)
        skills = []
        _skills = d.pop("skills")
        for skills_item_data in _skills:
            skills_item = UserSkillEntry.from_dict(skills_item_data)

            skills.append(skills_item)

        total = d.pop("total")

        skill_list_response = cls(
            skills=skills,
            total=total,
        )

        skill_list_response.additional_properties = d
        return skill_list_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
